#include "mesh_processor.h"
#include "util.h"
#include <QGenericMatrix>
#include <unordered_set>


static std::vector<HalfedgeHandle> findHoles(MyMesh &mesh) {
	std::vector<HalfedgeHandle> holes;
	std::set<HalfedgeHandle> ignore;
	for (HalfedgeHandle it : mesh.halfedges()) {
		if (mesh.is_boundary(it)
			&& ignore.find(it) == ignore.end()) {
			holes.push_back(it);
			ignore.insert(it);
			for (HalfedgeHandle it2 : HalfedgeLoopRange(mesh, it)) {
				ignore.insert(it2);
			}
		}
	}
	return holes;
}


MeshProcessor::MeshProcessor(const QString &path) {
	OpenMesh::IO::Options options;
	options.set(OpenMesh::IO::Options::VertexColor);
	if (!OpenMesh::IO::read_mesh(mesh, path.toUtf8().constData(), options)) {
		qWarning() << "Failed to read" << path;
		return;
	}
	holes = findHoles(mesh);
	fillHoles();
}


void fillHoleDumb(MyMesh &mesh, HalfedgeHandle &hole) {
	mesh.request_vertex_status();
	mesh.request_edge_status();
	mesh.request_face_status();

	Point center(0, 0, 0);
	size_t length = 0;
	for (HalfedgeHandle it : HalfedgeLoopRange(mesh, hole)) {
		VertexHandle vert = mesh.to_vertex_handle(it);
		center += mesh.point(vert);
		length++;
	}
	center /= length;
	VertexHandle center_handle = mesh.new_vertex_dirty(center);

	for (HalfedgeHandle it = hole;
		 mesh.to_vertex_handle(it) != center_handle;) {
		HalfedgeHandle next = mesh.next_halfedge_handle(it);
		mesh.add_face(mesh.from_vertex_handle(it),
					  mesh.to_vertex_handle(it),
					  center_handle);
		it = next;
	}
}

void MeshProcessor::fillHoles() {
	for (auto hole : holes) {
		fillHoleDumb(mesh, hole);
	}
}